#include <RF24Network.h>
#include <RF24.h>
#include <SPI.h>
#include <Wire.h>

RF24 radio(7,8);                // nRF24L01(+) radio attached using Getting Started board 

RF24Network network(radio);      // Network uses that radio
const uint16_t this_node = 00;    // Address of our node in Octal format ( 04,031, etc)

const int addr_nano = 8;         //addr I2C

struct nodos_a_central {          // Struct recibido de los nodos
  unsigned int nodo;              // nodo que envia
  unsigned int movimiento;        // 1 si hay movimiento
  unsigned int humo;              // 1 detección de humo
  unsigned int humedad;           // 1 si detección de humedad
};

unsigned int message;           
  /*  1 -> alarm enable
      2 -> alarm disabe
      3 -> lights always on
      4 -> 
      5 -> lights always off
  */

#define number_max_slaves 5

int max_number_retries = 10;
bool enviado[number_max_slaves + 1];
int retries[number_max_slaves + 1];
unsigned int message_to_slaves[number_max_slaves + 1];
char message_to_master[number_max_slaves + 1];

RF24NetworkHeader header;

#define noserial

void setup(void)
{
  #ifdef serial
  Serial.begin(9600);
  Serial.println("Test");
  #endif
 
  SPI.begin();
  radio.begin();
  radio.setPALevel(RF24_PA_MIN);
  radio.setDataRate(RF24_250KBPS);
  network.begin(/*channel*/ 90, /*node address*/ this_node);

  // Variables initialization
  for (int inode = 0; inode <= number_max_slaves; inode++) {
    enviado[inode] = true;
    //old_bit[inode] = 'a';
    retries[inode] = 0;
    message_to_slaves[inode] = 1;
    message_to_master[inode] = '0';
  }
  
  Wire.begin(addr_nano);
  Wire.onReceive(receiveEvent);
  Wire.onRequest(requestEvent);
  
  pinMode(2,OUTPUT);
  digitalWrite(2, LOW);
  
}

void loop(void){

  network.update();                  // Check the network regularly
  
  for (int inode = 1; inode <= number_max_slaves; inode++) {
    if (not enviado[inode]) {
      header.to_node = inode;
      message = message_to_slaves[inode];
      enviado[inode] = network.write(header,&message,sizeof(message));
      #ifdef serial
      Serial.print(inode);
      #endif
      if (enviado[inode]) {
        #ifdef serial
        Serial.println(".");
        Serial.print("Sent to ");
        Serial.print(inode);
        Serial.print(" ");
        Serial.println(message);
        #endif
        retries[inode] = 0;
        message_to_master[inode] = '*';
      } else {
        retries[inode]++;
        if (retries[inode] > max_number_retries) {
          enviado[inode] = true;
          retries[inode] = 0;
          message_to_master[inode] = '#';
        }
      }
    }
  }
  
  while ( network.available() ) {     // Is there anything ready for us?
    RF24NetworkHeader header;
    nodos_a_central nodos_a_central_t;
    network.read(header,&nodos_a_central_t,sizeof(nodos_a_central_t));
    if ((nodos_a_central_t.nodo > 0) and (nodos_a_central_t.nodo <= number_max_slaves)) {
      #ifdef serial
      Serial.print("Received packet: nodo: ");
      Serial.print(nodos_a_central_t.nodo);
      Serial.print(" movimiento: ");
      Serial.println(nodos_a_central_t.movimiento);
      #endif
      message_to_master[nodos_a_central_t.nodo] = '1';
      
      digitalWrite(2, HIGH);
      delay(250);
      digitalWrite(2, LOW);
    }
  }
}

// function that executes whenever data is received from master
void receiveEvent(int howMany) {
  int i = 0;
  #ifdef serial
  Serial.print("Received from ESP8266: ");
  #endif
  while (Wire.available() > 0) {
    char c = Wire.read();      /* receive byte as a character */
    #ifdef serial
    Serial.print(c);
    #endif
    if (isAlphaNumeric(c)) {
      // Messages for the network
      if ((i == 0) and (isDigit(c))) {
        message_to_master[i] = '*';
        for (int inode = 1; inode <= number_max_slaves; inode++) {
          if (enviado[inode]) {
            enviado[inode] = false;
            message_to_slaves[inode] = String(c).toInt();
          }
        }
      }
      // Messages to the slaves nodes with address <i>
      if ((i > 0) and (isDigit(c))) {
        if (enviado[i]) { 
          enviado[i] = false;
          message_to_slaves[i] = String(c).toInt();
        }
      }
      // Messages to the slaves nodes with address <i>
      // Change in minutes the time the lights is on
      if ((i > 0) and (isAlpha(c))) {
        if (enviado[i]) { 
          enviado[i] = false;
          message_to_slaves[i] = (int(c) - int('A')) + 100;
        }
      }
    }
    i = i + 1;
  }
  #ifdef serial
  Serial.println(" ");             /* to newline */
  #endif
  digitalWrite(2, LOW);
}
// function that executes whenever data is requested from master
void requestEvent() {   

  Wire.write(message_to_master);
  for (int inode = 0; inode <= number_max_slaves; inode++) {
      message_to_master[inode] = '0';
  }
  
}
