
#include <ESP8266WiFi.h>
#include <Wire.h>
#include "Gsender.h"

const char* ssid = "*** your ssid ***";
const char* password = "*** your password ***";
const char* to_email = "*** e-mail address where you will receive the message ***";
const int addr_nano = 8;
String lineal;
const unsigned long interval = 3000;
unsigned long last_check;
unsigned long last_check_newclient;

// Create an instance of the server
// specify the port to listen on as an argument
WiFiServer server(80);

#define number_max_slaves 5

bool hay_alarmas;

char mensajes_a_nodos[number_max_slaves + 1]; 
char network_state[number_max_slaves + 1];
char time_lights[number_max_slaves + 1];
int alarm_counter[number_max_slaves + 1]; 

const char const_network_alarm_connected = '1';
const char const_network_alarm_disconnected = '2';
const char const_network_lights_when_movement = '4';
const char const_network_lights_always_on = '3';

const char const_station_lights_when_movement = '4';
const char const_station_lights_always_on = '3';

const char const_movement_detected = '1';

const char const_local_time_on_1m = '6';
const char const_local_time_on_2m = '7';

const char const_time_1minute = 'B'; // ASC('B') - ASC('A')
const char const_time_2minute = 'C'; // ASC('C') - ASC('A')

void setup() {
  Serial.begin(9600);
  delay(10);

  last_check = millis();

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());

  Wire.begin(D1, D2);           /* join i2c bus with SDA=D1 and SCL=D2 of NodeMCU */

  hay_alarmas = false;
  mensajes_a_nodos[0] = const_network_alarm_connected;
  network_state[0] = const_network_alarm_connected;
  time_lights[0] = const_network_lights_when_movement;
  for (int inode = 1; inode <= number_max_slaves;inode++) {
    mensajes_a_nodos[inode] = '*';
    alarm_counter[inode] = 0;
    time_lights[inode] = const_local_time_on_2m;
    network_state[inode] = const_station_lights_when_movement;
  }
}

boolean SendEmail(String subject, String message) {
  Gsender *gsender = Gsender::Instance();    // Getting pointer to class instance
  if(gsender->Subject(subject)->Send(to_email, message)) {
      Serial.println("Message send.");
      return true;
  } else {
      Serial.print("Error sending message: ");
      Serial.println(gsender->getError());
      return false;
  }
}

void loop() {

  unsigned long now = millis();
  if (now-last_check >= interval) {
    int i = 0;
    Wire.requestFrom(addr_nano, number_max_slaves + 1);
    while(Wire.available() > 0) {
      char c = Wire.read();
      if (c == '*') {                             // Acknoledge. The message has been sent by the NANO
        mensajes_a_nodos[i] = '*';
        if (network_state[i] == '#') network_state[i] = const_station_lights_when_movement;
        if (time_lights[i] == '#') time_lights[i] = const_local_time_on_2m;
      }
      if (c == '#') {                             // Slave sation is not online
        mensajes_a_nodos[i] = '*';
        network_state[i] = '#';
        time_lights[i] = '#';
      }
      if (c == const_movement_detected) {
        alarm_counter[i]++;
        hay_alarmas = true;
        SendEmail("¡Movement detected in Node" + String(i) + "!", "¡Movement detected in Node" + String(i) + "!");
      }
      i = i + 1;
      Serial.print(c);
    }
    Serial.println();
    last_check = now;

    Wire.beginTransmission(addr_nano);
    Wire.write(mensajes_a_nodos);
    Wire.endTransmission();      
  }

  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }

  // Wait until the client sends some data
  Serial.println("new client");
  last_check_newclient = millis();
  now = millis();
  while (!client.available() and (now-last_check_newclient < interval)) {
    delay(100);
    now = millis();
  }
  if (now-last_check_newclient >= interval) return;

  String linea1 = client.readStringUntil('r');
  Serial.println(linea1);
  
  if (linea1.indexOf("ALARON")>0) 
  {
      mensajes_a_nodos[0] = const_network_alarm_connected;
      network_state[0] = const_network_alarm_connected;
  }
  if (linea1.indexOf("ALAROFF")>0)
  {
     mensajes_a_nodos[0] = const_network_alarm_disconnected;
     network_state[0] = const_network_alarm_disconnected;
  }
  if (linea1.indexOf("RESALARM")>0)
  {
    for (int inode = 1; inode <= number_max_slaves;inode++) alarm_counter[inode] = 0;
    hay_alarmas = false;
  }

  //LIGHTING
  if (linea1.indexOf("NETON")>0)
  {
    mensajes_a_nodos[0] = const_network_lights_always_on;
    time_lights[0] = const_network_lights_always_on;
  }
  if (linea1.indexOf("NETOFF")>0)
  {
    mensajes_a_nodos[0] = const_network_lights_when_movement;
    time_lights[0] = const_network_lights_when_movement;
  }

  for (int inode = 1; inode <= number_max_slaves;inode++) {
    if (linea1.indexOf("NODE" + String(inode) + "1M")>0)
    {
      mensajes_a_nodos[inode] = const_time_1minute;
      time_lights[inode] = const_local_time_on_1m;
    }      
    if (linea1.indexOf("NODE" + String(inode) + "2M")>0)
    {
      mensajes_a_nodos[inode] = const_time_2minute;
      time_lights[inode] = const_local_time_on_2m;
    }
    if (linea1.indexOf("NODE" + String(inode) + "MOV")>0)
    {
      mensajes_a_nodos[inode] = const_station_lights_when_movement;
      network_state[inode] = const_station_lights_when_movement;
    }  
    if (linea1.indexOf("NODE" + String(inode) + "ON")>0)
    {
      mensajes_a_nodos[inode] = const_station_lights_always_on;
      network_state[inode] = const_station_lights_always_on;
    }
  }

  client.flush();
  Serial.println("Enviando respuesta...");   
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println("Connection: close");// La conexión se cierra después de finalizar de la respuesta
  client.println();
  client.println("<!DOCTYPE HTML>");
  client.println("<html>");

  //STATE SECTION
  client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><big><code><span style=""""font-size:72px;"""">STATE</span></code></big></span></p>");
  if (network_state[0] == const_network_alarm_connected) client.println("<p><span style=""""font-size:35px;""""><big><code><img alt=""""yes"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/thumbs_up.gif"""" title=""""yes"""" width=""""60"""" />ALARM CONNECTED</code></big></span></p>");
  if (network_state[0] == const_network_alarm_disconnected) client.println("<p><span style=""""font-size:35px;""""><big><code><img alt=""""no"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/thumbs_down.gif"""" title=""""no"""" width=""""60"""" />ALARM DISCONNECTED</code></big></span></p>");

  //COUNTERS ALARM SECTION
  client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><big><code><span style=""""font-size:72px;"""">ALARMS COUNTERS</span></code></big></span></p>");
  if (hay_alarmas == false) {
    client.println("<p><span style=""""font-size:35px;""""><big><code><img alt=""""smiley"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/regular_smile.gif"""" title=""""smiley"""" width=""""60"""" />NO ALARMS</code></big></span></p>");
  } else {
    for (int inode = 1; inode <= number_max_slaves;inode++) {
      if (alarm_counter[inode] > 0) {
        client.print("<p><span style=""""font-size:35px;""""><big><code><img alt=""""surprise"""" height=""""60"""" src=""""http://bestonlinehtmleditor.com/ckeditor/plugins/smiley/images/omg_smile.gif"""" title=""""surprise"""" width=""""60"""" />Node ");
        client.print(inode);
        client.print(" :  ");
        client.print(alarm_counter[inode]);
        client.println("</code></big></span></p>");        
      }
    }
  };

  //COMMANDS SECTION
  client.println("<p style=""""text-align: center;""""><span style=""""background-color:Lime;""""><big><code><span style=""""font-size:72px;"""">ALARM</span></code></big></span></p>");
  client.println("<div style='text-align:center;'>");
  if (network_state[0] == const_network_alarm_disconnected) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor1"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?ALARON';"""" value=""""CONNECT_ALARM"""" /></p>");
  if (network_state[0] == const_network_alarm_connected) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor2"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?ALAROFF';"""" value=""""DISCONNECT_ALARM"""" /></p>");
  client.println("<p style=""""text-align: center;""""><input name=""""btn_valor3"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?RESALARM';"""" value=""""RESET_COUNTERS"""" /></p>");

  //LIGHTING SECTION
  client.println("<p style=""""text-align:left;""""><span style=""""background-color:Lime;""""><big><code><span style=""""font-size:72px;"""">LIGHTING</span></code></big></span></p>");
  if (time_lights[0] == const_station_lights_when_movement) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?NETON';"""" value="""" NETWORK_ALWAYS_ON """" /></p>");
  if (time_lights[0] == const_station_lights_always_on) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?NETOFF';"""" value="""" NETWORK_WHEN_MOVEMENT """" /></p>");

  for (int inode = 1; inode <= number_max_slaves;inode++) {
    if (network_state[inode] == const_station_lights_when_movement) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?NODE" + String(inode) + "ON';"""" value="""" Node" + String(inode) + "_always_ON """" /></p>");
    if (network_state[inode] == const_station_lights_always_on) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?NODE" + String(inode) + "MOV';"""" value="""" Node" + String(inode) + "_mov """" /></p>");
    if (time_lights[inode] == const_local_time_on_2m) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?NODE" + String(inode) + "1M';"""" value="""" Node" + String(inode) + "_ON_1_minute """" /></p>");
    if (time_lights[inode] == const_local_time_on_1m) client.println("<p style=""""text-align: center;""""><input name=""""btn_valor"""" style=""""font-size:60px;"""" type=""""button"""" onclick=""""location.href='./?NODE" + String(inode) + "2M';"""" value="""" Node" + String(inode) +"_ON_2_minute """" /></p>");
  }
    
  client.println("</div>");
  client.println("</html>");
  
  delay(1);
  Serial.println("Client disconnected");

  // The client will actually be disconnected
  // when the function returns and 'client' object is detroyed
}
